# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from ... import decorators


@decorators.OGClass(':contact_data', (
        'street_address', 'locality', 'region', 'postal_code', 'country_name',
        'email', 'phone_number', 'fax_number', 'website'
))
class OGContactData(models.Model):
    """
    URI: https://developers.facebook.com/blog/post/2013/06/05/platform-updates--new-open-graph-business-object-type/
    _________________________________________

    Entities:

    ...:contact_data:street_address  -> Address of the object.
    ...:contact_data:locality        -> Locality of the object.
    ...:contact_data:region          -> Region of the object.
    ...:contact_data:postal_code     -> Postal code of the object
    ...:contact_data:country_name    -> Country of the object
    ...:contact_data:email           -> Email of the object
    ...:contact_data:phone_number    -> Phone number of the object
    ...:contact_data:fax_number      -> Fax of the object
    ...:contact_data:website         -> Main page of this object website

    _________________________________________
    """

    class Meta:
        db_table = 'contact_data'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('contact data')
        verbose_name_plural = _('contact data')

    street_address = models.CharField(_('street address'), max_length=255, null=True)
    locality = models.CharField(_('locality'), max_length=255, null=True)
    region = models.CharField(_('region'), max_length=255, null=True)
    postal_code = models.CharField(_('postal code'), max_length=32, null=True)
    country_name = models.CharField(_('country name'), max_length=64, null=True)
    email = models.EmailField(_('email address'), null=True)
    phone_number = models.CharField(_('phone number'), max_length=16, null=True)
    fax_number = models.CharField(_('fax number'), max_length=16, null=True)
    website = models.URLField(_('website url'), null=True)
