import json
from pathlib import Path
from typing import Generator, Tuple
from django.conf import settings
from django.utils.translation import gettext_noop as _n

fpath = Path(__file__).resolve().parent.parent.parent.parent / 'data' / 'currencies.json'
prefix = "OPENGRAPH_PRODUCT_CURRENCY"
_fullname = settings.get(f"{prefix}_FULLNAME", False)

with open(settings.get(f"{prefix}_FILEPATH", fpath), 'r', -1, 'utf8') as file:
    data = json.load(file)

del prefix, fpath


class Currency:
    @property
    def code(self) -> str:
        return self._code

    @property
    def name(self) -> str:
        return self._name

    @property
    def id(self) -> int:
        x = self._code.encode('utf8')
        return (x[0]-0x40 & 0x1f) | ((x[1]-0x40 & 0x1f) << 5) | ((x[2]-0x40 & 0x1f) << 10)

    def __int__(self) -> int:
        return self.id

    def __str__(self) -> str:
        return self.code

    def __init__(self, code: str, name: str):
        self._code = code
        self._name = name


class Currencies:

    class NotFound(ValueError):
        pass

    @property
    def _choices(self):
        if not self.__sorted:
            self.__sorted = [i for i in self.__dict__.values() if isinstance(i, Currency)]
            self.__sorted.sort(key=lambda y: y.code)
        return self.__sorted

    @property
    def choices(self) -> Tuple[Tuple[int, str], ...]:
        return tuple([(i.id, _n(i.name if _fullname else i.code)) for i in self._choices])

    def currencies(self) -> Generator:
        for i in self._choices:
            yield i.code

    def find_currency(self, id: int) -> Currency:
        for i in self._choices:
            if i.id == id:
                return i
        raise self.NotFound(f'currency id {id}')

    def __getitem__(self, key: str) -> Currency:
        for i in self._choices:
            if i.code == key:
                return i
        raise KeyError(key)

    def __call__(self, *args, **kwargs):
        return self.__class__(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        self.__sorted = None
        for k, v in kwargs.items():
            setattr(self, k.upper(), Currency(k.upper(), v))


Currencies = Currencies(**data)
del data
