# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from ... import decorators


@decorators.OGClass(':site', ('!username', 'id'))
class TwitterCardSite(models.Model):
    """
    URI: https://developers.facebook.com/blog/post/2013/06/05/platform-updates--new-open-graph-business-object-type/
    _________________________________________

    Entities:

    ...:site    -> @username of website.
    ...:site:id -> Same as ...:site, but the user's Twitter ID.

    _________________________________________
    """

    class Meta:
        db_table = 'profile'
        app_label = settings.get('TWITTER_CARD_META_APP_LABEL', 'twitter_card')
        verbose_name = _('profile meta information')
        verbose_name_plural = _('profiles meta information')

    username = models.CharField(_('username'), max_length=32, unique=True)
    id = models.CharField(_('id'), max_length=32, null=True)

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith(cls.og_name()) and not v.startswith('@'):
            v = f'@{v}'
        yield k, str(v)


@decorators.OGClass(':creator', ('!username', 'id'))
class TwitterCardCreator(TwitterCardSite):

    class Meta:
        proxy = True
